%| demo2_str_cvx_case2.m
%| demo 2
%|      demonstrates a strongly convex quadratic example in Section 6.1.2 of [KF17a]
%|
%| [KF17a] D. Kim, J.A. Fessler
%| "Adaptive restart of the optimized gradient method for convex optimization,"
%| Arxiv:1703.04641, (2017)
%|
%| Copyright 2017-3-31, Donghwan Kim and Jeff Fessler, University of Michigan

%clear all;
%close all;

color_peach = [251 111 66] ./ 255; % for plot

% generate problem
A = [0.01 0; 0 1];
b = zeros(2,1);

% anonymous function for computing first-order information
f_cost = @(x) x'*(1/2*A*x - b);
f_grad = @(x) A*x - b;
% function parameters
f_L = max(eig(A));
f_mu = 0; % zero is a lower bound

% optimum
xopt = [0 0]';
fopt = f_cost(xopt);

% initial
x0 = [0.2 1]';

% number of iterations
niter = 150;

% GM
mom = 0; restart = 0; bsig = 1; seq = 1;
[xs, fs] = gm_restart(x0, f_cost, f_grad, f_L, f_mu, mom, restart, bsig, niter, seq);

% FGM w/ gradient restart
mom = 1; restart = 2;
[xsf_gr, fsf_gr, rsf_gr] = gm_restart(x0, f_cost, f_grad, f_L, f_mu, mom, restart, bsig, niter, seq);

% OGM w/ gradient restart
mom = 2; seq = 1; % primary sequence
[xso1_gr_sig1, fso1_gr_sig1, rso1_gr_sig1] = ...
	gm_restart(x0, f_cost, f_grad, f_L, f_mu, mom, restart, bsig, niter, seq);
seq = 2; % secondary sequence
[xso2_gr_sig1, fso2_gr_sig1, rso2_gr_sig1] = ...
	gm_restart(x0, f_cost, f_grad, f_L, f_mu, mom, restart, bsig, niter, seq);

% OGM w/ gradient restart + gradient gamma decreasing (bsig = 0.8)
bsig = 0.8; seq = 1; % primary sequence
[xso1_gr_sig08, fso1_gr_sig08, rso1_gr_sig08] = ...
	gm_restart(x0, f_cost, f_grad, f_L, f_mu, mom, restart, bsig, niter, seq);
seq = 2; % secondary sequence
[xso2_gr_sig08, fso2_gr_sig08, rso2_gr_sig08] = ...
	gm_restart(x0, f_cost, f_grad, f_L, f_mu, mom, restart, bsig, niter, seq);

% OGM w/ gradient restart + gradient gamma decreasing (bsig = 0.5)
bsig = 0.5; seq = 1; % primary sequence
[xso1_gr_sig05, fso1_gr_sig05, rso1_gr_sig05] = ...
	gm_restart(x0, f_cost, f_grad, f_L, f_mu, mom, restart, bsig, niter, seq);
seq = 2; % secondary sequence
[xso2_gr_sig05, fso2_gr_sig05, rso2_gr_sig05] = ...
	gm_restart(x0, f_cost, f_grad, f_L, f_mu, mom, restart, bsig, niter, seq);

% display
%figure(1);
iter = 0:niter;
f = semilogy( ...
	iter, fs - fopt, 'k-', ...
	iter, fsf_gr - fopt, 'c--', ...
	iter, fso1_gr_sig1 - fopt, 'm--', ... 
	iter(30), fso2_gr_sig1(30) - fopt, 'm+-', ...
	iter, fso1_gr_sig08 - fopt, 'y-.', ...
	iter(7), fso2_gr_sig08(7) - fopt, 'y*-', ...
	iter, fso1_gr_sig05 - fopt, 'g:', ...
	iter(4), fso2_gr_sig05(4) - fopt, 'gx-');
hold on
fa = semilogy( ...
	iter, fso2_gr_sig1 - fopt, 'm-', ...
	iter, fso2_gr_sig08 - fopt, 'y-', ...
        iter, fso2_gr_sig05 - fopt, 'g-');
hold off
l = {'GM', 'FGM-GR', ...
	'OGM-GR-GD$\gamma$($\bar{\sigma} = 1.0$)-prim.', ...
	'OGM-GR-GD$\gamma$($\bar{\sigma} = 1.0$)-sec.', ...
	'OGM-GR-GD$\gamma$($\bar{\sigma} = 0.8$)-prim.', ...
	'OGM-GR-GD$\gamma$($\bar{\sigma} = 0.8$)-sec.', ...
	'OGM-GR-GD$\gamma$($\bar{\sigma} = 0.5$)-prim.', ...
	'OGM-GR-GD$\gamma$($\bar{\sigma} = 0.5$)-sec.'};

set(f(5), 'color', color_peach);
set(f(6), 'color', color_peach);
set(fa(2), 'color', color_peach);

xlim([0 niter])
ylim([10^(-15) f_cost(x0)-fopt])

ms = 11; gf = 16; ff= 14;
set(f, 'MarkerSize', ms, {'LineWidth'}, {1 3 5 5 3 3 3 3}');
set(fa, 'MarkerSIze', ms, {'LineWidth'}, {5 3 3}');
set(gca, 'fontsize', gf);
legend(f, l, 'Location', 'NorthEast', 'fontsize', ff, 'Interpreter', 'latex');
xlabel('Iteration ($k$)', 'Interpreter', 'latex');
ylabel('$f(y_k) - f(x_*)$ or $f(x_k) - f(x_*)$', 'Interpreter', 'latex');

% save fig
%print('-depsc', '../fig/fig,demo2_str_cvx_case2.eps')
